/* ============================================================================================================================================= /
 * PROJECT:         Headphone Amp 
 * FILE:            Main.c
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F26K22
 * CREATED:         11/08/2017
 * UPDATED:
 *
 * DESCRIP:         
 *
 * ============================================================================================================================================= /
 * Change History:
 * Rev.             Date                Changes
 /* ============================================================================================================================================ *

 /* I/O PIC18F46K22 ============================================================================================================================ /
 * RA0:     IO_1
 * RA1:     IO_2
 * RA2:     IO_3
 * RA3:     IO_4
 * RA4:     IO_5
 * RA5:     IO_6
 * RA6:     REL_HEADPHONE
 * RA7:     D_5V_EN
 * RB0:     IR_IN
 * RB1:     SEL_IN_SW
 * RB2:     ENC_SW
 * RB3:     ENC_A
 * RB4:     ENC_B
 * RB5:     OV_CHK
 * RC0:     PGA_CS
 * RC1:     PGA_ZCEN
 * RC2:     HP_CHK //0 CUFFIA NON CONNESSA
 * RC3:     SPI_SCL
 * RC4:     OLED_CS
 * RC5:     SPI_SDO
 * RC6:     OLED_DC     
 * RC7:     OLED_RST
 */


 /* CONFIGURATION BITS ========================================================================================================================= */ 
// CONFIG1H
#pragma config FOSC = INTIO67   // Oscillator Selection bits (Internal oscillator block)
#pragma config PLLCFG = OFF     // 4X PLL Disabled
#pragma config PRICLKEN = ON    // Primary clock enable bit (Primary clock enabled)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable bit (Fail-Safe Clock Monitor disabled)
#pragma config IESO = OFF       // Internal/External Oscillator Switchover bit (Oscillator Switchover mode disabled)

// CONFIG2L
#pragma config PWRTEN = OFF     // Power-up Timer Enable bit (Power up timer disabled)
#pragma config BOREN = OFF      // Brown-out Reset Enable bits (Brown-out Reset always disabled)
#pragma config BORV = 190       // Brown Out Reset Voltage bits (VBOR set to 1.90 V nominal)

// CONFIG2H
#pragma config WDTEN = OFF       // Watchdog Timer Enable bits (WDT is always disabled.)
#pragma config WDTPS = 32768    // Watchdog Timer Postscale Select bits (1:32768)

// CONFIG3H
#pragma config CCP2MX = PORTC1  // CCP2 MUX bit (CCP2 input/output is multiplexed with RC1)
#pragma config PBADEN = OFF     // PORTB A/D Enable bit (PORTB<5:0> pins are configured as analog input channels on Reset)
#pragma config CCP3MX = PORTB5  // P3A/CCP3 Mux bit (P3A/CCP3 input/output is multiplexed with RB5)
#pragma config HFOFST = ON      // HFINTOSC Fast Start-up (HFINTOSC output and ready status are not delayed by the oscillator stable status)
#pragma config T3CMX = PORTC0   // Timer3 Clock input mux bit (T3CKI is on RC0)
#pragma config P2BMX = PORTB5   // ECCP2 B output mux bit (P2B is on RB5)
#pragma config MCLRE = EXTMCLR  // MCLR Pin Enable bit (MCLR pin enabled, RE3 input pin disabled)

// CONFIG4L
#pragma config STVREN = ON      // Stack Full/Underflow Reset Enable bit (Stack full/underflow will cause Reset)
#pragma config LVP = ON         // Single-Supply ICSP Enable bit (Single-Supply ICSP disabled)
#pragma config XINST = OFF      // Extended Instruction Set Enable bit (Instruction set extension and Indexed Addressing mode disabled (Legacy mode))

// CONFIG5L
#pragma config CP0 = OFF        // Code Protection Block 0 (Block 0 (000800-003FFFh) not code-protected)
#pragma config CP1 = OFF        // Code Protection Block 1 (Block 1 (004000-007FFFh) not code-protected)
#pragma config CP2 = OFF        // Code Protection Block 2 (Block 2 (008000-00BFFFh) not code-protected)
#pragma config CP3 = OFF        // Code Protection Block 3 (Block 3 (00C000-00FFFFh) not code-protected)

// CONFIG5H
#pragma config CPB = OFF        // Boot Block Code Protection bit (Boot block (000000-0007FFh) not code-protected)
#pragma config CPD = OFF        // Data EEPROM Code Protection bit (Data EEPROM not code-protected)

// CONFIG6L
#pragma config WRT0 = OFF       // Write Protection Block 0 (Block 0 (000800-003FFFh) not write-protected)
#pragma config WRT1 = OFF       // Write Protection Block 1 (Block 1 (004000-007FFFh) not write-protected)
#pragma config WRT2 = OFF       // Write Protection Block 2 (Block 2 (008000-00BFFFh) not write-protected)
#pragma config WRT3 = OFF       // Write Protection Block 3 (Block 3 (00C000-00FFFFh) not write-protected)

// CONFIG6H
#pragma config WRTC = OFF       // Configuration Register Write Protection bit (Configuration registers (300000-3000FFh) not write-protected)
#pragma config WRTB = OFF       // Boot Block Write Protection bit (Boot Block (000000-0007FFh) not write-protected)
#pragma config WRTD = OFF       // Data EEPROM Write Protection bit (Data EEPROM not write-protected)

// CONFIG7L
#pragma config EBTR0 = OFF      // Table Read Protection Block 0 (Block 0 (000800-003FFFh) not protected from table reads executed in other blocks)
#pragma config EBTR1 = OFF      // Table Read Protection Block 1 (Block 1 (004000-007FFFh) not protected from table reads executed in other blocks)
#pragma config EBTR2 = OFF      // Table Read Protection Block 2 (Block 2 (008000-00BFFFh) not protected from table reads executed in other blocks)
#pragma config EBTR3 = OFF      // Table Read Protection Block 3 (Block 3 (00C000-00FFFFh) not protected from table reads executed in other blocks)

// CONFIG7H
#pragma config EBTRB = OFF      // Boot Block Table Read Protection bit (Boot Block (000000-0007FFh) not protected from table reads executed in other blocks)

/* CONSTANTI ==================================================================================================================================== */
#define XTAL_FREQ           16                                                          // definisco la frequenza per Libreria Delay
#define ON  1
#define OFF 0
#define CONNECTED           1
#define DISCONNECTED        0
#define BOOT                2
#define WAKE                3


// modalit
#define MAIN                0x00
#define P_DOWN              0x01
#define SETUP               0x02
#define IR_IDLE             0x10
#define IR_INCOMING         0x11
#define IR_RECEIVED         0x12
#define IR_DECODED          0x13
#define MAIN_MENU           0x00
#define AUDIO_MENU          0x01
#define SYSTEM_MENU         0x02
#define TRIM_MENU           0x03
#define NAME_MENU           0x04    
#define INFO                0x05

/* PERSONALIZZAZIONI ============================================================================================================================= */
/*
 * Limite volume master PGA2320 a +6dB (204)
 * Trim massimo in ingresso +25.5db (51)
*/
#define LIMIT_MASTER_VOL    204
#define LIMIT_TRIM          51

/*
 * Velocity control encoder
 * modificare per settare le soglie delle velocit di rotazione
*/
#define VELO_0              30
#define VELO_1              15
#define VELO_2              8
#define VELO_3              3
                
/*
 * Visualizzazione su display del guadagno reale comprensivo del guadagno applicato dal TPA6120 (+6dB) o del guadagno applicato solo dal PGA2320
 * Decommentare la selezione
*/
#define REAL_GAIN       
//#define PGA2320_GAIN

/* 
 * Numero di IO usati.
 * Per aggiungere altri io incrementare fino a 6
*/
#define IO_USED             3

/* 
 * Varianti IR implementate
 * Incrementare se si implementano nuove varianti IR, in menu il valore si aggiorna automaticamente con questo parametro
*/
#define MAX_REMOTE_TYPE     4

/* 
 * Anti pressione multipla per Varianti IR 2-3-4
 * Timer 0: interrupt ogni 100 ms, MSEC_ANTIBOUNCE: 1 = 100ms, 5 = 500ms ecc
 * Tempo tra una pressione e l'altra dei tasti volume, per gli altri tasti il parametro viene moltiplicato per 3
*/
#define MSEC_ANTIBOUNCE     2
#define TMR0_PRELOAD        15538

/* 
 * Rel cuffie installato, commentare se non installato
*/
#define REL_HEADPHONE_INSTALLED

/* 
 * Abilitazione invio dati di debug via uart
*/
//#define UART_DEBUG

/* 
 * Connessioni PIC
*/
#define ENC_A               PORTBbits.RB4                                               // invertire i 2 ingressi se l'encoder risponde al contrario
#define ENC_B               PORTBbits.RB3  
#define ENC_SW              (unsigned char *)&PORTB, 0b00000100
#define SEL_IN_SW           (unsigned char *)&PORTB, 0b00000010
#define IR_IN               PORTBbits.RB0
#define OV_CHK              PORTBbits.RB5
#define HP_CHK              PORTCbits.RC2
#define ENC_SW_DIR          PORTBbits.RB2
#define SEL_IN_SW_DIR       PORTBbits.RB1

#define RELE_IO             LATA     
#define REL_IN_0            LATAbits.LATA0
#define REL_IN_1            LATAbits.LATA1
#define REL_IN_2            LATAbits.LATA2
#define REL_IN_3            LATAbits.LATA3
#define REL_IN_4            LATAbits.LATA4
#define REL_IN_5            LATAbits.LATA5
#define REL_HEADPHONE       LATAbits.LATA6
#define D_5V_EN             LATAbits.LATA7    


/* ============================================================================================================================================= /
 * FUNCTION:        DefineIO
 * Overview:        Definizione e inizializzazione ingressi e uscite
/* ============================================================================================================================================ */

void Define_IO(void) {
    LATA =      0b00000000;                                                             
    ANSELA =    0b00000000;
    TRISA =     0b00000000;
    LATB =      0b00000000;                                                             // Reset Latch uscite
    ANSELB =    0b00000000;
    TRISB =     0b11111111;
    LATC =      0b00010001;                                                             // Reset Latch uscite imposto subito i CS SPI a 1
    ANSELC =    0b00000000;
    TRISC =     0b00000100;
}

/* ============================================================================================================================================= /
 * FUNCTION:        Module_settings
 * Overview:        Configurazioni dei moduli interni
/* ============================================================================================================================================ */

void Module_settings(void) {
    INTCON2bits.RBPU = 0;                                                               // abilito resistenze pull up su PORT, 0: tutte le resistenze abilitate
                                                                                        //                                     1: tutte le resistenze disabilitate
    WPUB = 0b00011110;                                                                  // abilitazione singole resistenze di pullup su PORTB
    
    OSCCONbits.IRCF0 = 1;                                                               // oscillatore interno 16 MHz
    OSCCONbits.IRCF1 = 1;    
    OSCCONbits.IRCF2 = 1;    
    OSCTUNEbits.PLLEN = 0;                                                              // 
    
    CCP1CONbits.CCP1M0 = 0;                                                             // disabilito tutti i comparatori
    CCP1CONbits.CCP1M1 = 0;
    CCP1CONbits.CCP1M2 = 0;
    CCP1CONbits.CCP1M3 = 0;

    CCP2CONbits.CCP2M0 = 0;
    CCP2CONbits.CCP2M1 = 0;
    CCP2CONbits.CCP2M2 = 0;
    CCP2CONbits.CCP2M3 = 0;
    
    CCP4CONbits.CCP4M = 0;

/* INTERRUPT GLOBALE e PRIORITA'*/
    RCONbits.IPEN = 1;
    INTCONbits.GIEH = 1;
    INTCONbits.GIEL = 1;         
    
// TIMER 0, Configurazioni per avere 1 interrupt ogni 100ms = (65536-preload)*(4/fosc)*prescaler = (65536-15536)*(4/16000000)*8
    T0CONbits.T0PS0 = 0;                                                                // prescaler 1/8
    T0CONbits.T0PS1 = 1;                                                                
    T0CONbits.T0PS2 = 0;
    T0CONbits.PSA = 0;                                                                  // prescaler assegnato
    T0CONbits.T0CS = 0;                                                                 // da clock interno fosc/4
    T0CONbits.T08BIT = 0;                                                               // 16 BIT
//    TMR0 = tmr0_preload;                                                                       // preload timer 0

    T0CONbits.TMR0ON = 0; 
    
    INTCON2bits.TMR0IP = 0;                                                             // priorit bassa
    INTCONbits.TMR0IF = 0;
    INTCONbits.TMR0IE = 1;
    
// TIMER 2, Configurazioni per avere 1 interrupt ogni 4.5ms (220Hz) Preload = 30   -> 2.5ms (400Hz) preload = 130
    #define TMR2_PRELOAD    30
    T2CONbits.T2CKPS0 = 0;                                                              // prescaler 16
    T2CONbits.T2CKPS1 = 1;
    T2CONbits.T2OUTPS0 = 0;                                                             // postscaler 5
    T2CONbits.T2OUTPS1 = 0;
    T2CONbits.T2OUTPS2 = 1;
    T2CONbits.T2OUTPS3 = 0;
    PR2 = 255;
    TMR2 = TMR2_PRELOAD;
    
    T2CONbits.TMR2ON = ON;
    IPR1bits.TMR2IP = 0;     // priorit bassa
    PIR1bits.TMR2IF = 0;
    PIE1bits.TMR2IE = 1;
    
// INT0 (IR) sempre priorit alta
    INTCON2bits.INTEDG0 = 0; // falling edge
    INTCONbits.INT0IF = 0;
    INTCONbits.INT0IE = 1;    
    
// INT1 (Pulsante input)
    INTCON2bits.INTEDG1 = 0; // falling edge
    INTCON3bits.INT1F = 0;
    INTCON3bits.INT1IE = 0;
    INTCON3bits.INT1IP = 0;
    
// INT2 (pulsante encoder)
    INTCON2bits.INTEDG2 = 0; // falling edge
    INTCON3bits.INT2IF = 0;
    INTCON3bits.INT2IE = 0;
    INTCON3bits.INT2IP = 0;  // bassa priorit
    
// INTERRUPT ON CHANGE RB5 (over voltage)
    INTCONbits.RBIF = 0;
    INTCONbits.RBIE = 0;
    INTCON2bits.RBIP = 1;
    IOCBbits.IOCB4 = 0;
    IOCBbits.IOCB5 = 1;
    IOCBbits.IOCB6 = 0;
    IOCBbits.IOCB7 = 0;
        
// POWER MANAGEMENT
    OSCCONbits.IDLEN = 1;                                                               // modalit IDLE (periferiche attive)
    
#ifdef UART_DEBUG
    PMD0bits.UART2MD = 0;                                                               // disabilito i moduli non utilizzati
#else
    PMD0bits.UART2MD = 1;
#endif
    
    PMD0bits.TMR1MD = 1;
 //   PMD0bits.TMR2MD = 1;                                                              // timer per encoder gestito in funzione Power()
    PMD0bits.TMR3MD = 1;
    PMD0bits.TMR4MD = 1;
    PMD0bits.TMR5MD = 1;
    PMD0bits.TMR6MD = 1;
    
    PMD1bits.CCP1MD = 1;
    PMD1bits.CCP2MD = 1;
    PMD1bits.CCP3MD = 1;
    PMD1bits.CCP4MD = 1;
    PMD1bits.CCP5MD = 1;
    PMD1bits.MSSP2MD = 1;

    PMD2bits.CMP1MD = 1;
    PMD2bits.CMP2MD = 1;
    PMD2bits.CTMUMD = 1;
    PMD2bits.ADCMD = 1; 
}